@@ -1,8 +1,8 @@
 #![feature(test)]
 
-use sysinfo::get_current_pid;
-use sysinfo::{DiskExt, SystemExt};
+extern crate test;
 
+#[cfg(feature = "system")]
 #[bench]
 fn bench_new(b: &mut test::Bencher) {
     b.iter(|| {
@@ -10,6 +10,7 @@ fn bench_new(b: &mut test::Bencher) {
     });
 }
 
+#[cfg(feature = "system")]
 #[bench]
 fn bench_new_all(b: &mut test::Bencher) {
     b.iter(|| {
@@ -17,6 +18,7 @@ fn bench_new_all(b: &mut test::Bencher) {
     });
 }
 
+#[cfg(feature = "system")]
 #[bench]
 fn bench_refresh_all(b: &mut test::Bencher) {
     let mut s = sysinfo::System::new_all();
@@ -26,43 +28,44 @@ fn bench_refresh_all(b: &mut test::Bencher) {
     });
 }
 
-#[bench]
-fn bench_refresh_system(b: &mut test::Bencher) {
-    let mut s = sysinfo::System::new_all();
-
-    s.refresh_system();
-    b.iter(move || {
-        s.refresh_system();
-    });
-}
-
+#[cfg(feature = "system")]
 #[bench]
 fn bench_refresh_processes(b: &mut test::Bencher) {
     let mut s = sysinfo::System::new();
 
-    s.refresh_processes(); // to load the whole processes list a first time.
+    s.refresh_processes(sysinfo::ProcessesToUpdate::All, true); // to load the whole processes list a first time.
     b.iter(move || {
-        s.refresh_processes();
+        s.refresh_processes(sysinfo::ProcessesToUpdate::All, true);
     });
 }
 
+#[cfg(feature = "system")]
+#[bench]
+fn bench_first_refresh_processes(b: &mut test::Bencher) {
+    b.iter(move || {
+        let mut s = sysinfo::System::new();
+        s.refresh_processes(sysinfo::ProcessesToUpdate::All, true);
+    });
+}
+
+#[cfg(feature = "system")]
 #[bench]
 fn bench_refresh_process(b: &mut test::Bencher) {
     let mut s = sysinfo::System::new();
 
     s.refresh_all();
     // to be sure it'll exist for at least as long as we run
-    let pid = get_current_pid().expect("failed to get current pid");
+    let pid = sysinfo::get_current_pid().expect("failed to get current pid");
     b.iter(move || {
-        s.refresh_process(pid);
+        s.refresh_processes(sysinfo::ProcessesToUpdate::Some(&[pid]), true);
     });
 }
 
 #[bench]
 fn bench_refresh_disk(b: &mut test::Bencher) {
-    let mut s = sysinfo::System::new_all();
+    let mut disks = sysinfo::Disks::new_with_refreshed_list();
 
-    let disks = s.disks_mut();
+    let disks = disks.list_mut();
     let disk = &mut disks[0];
     b.iter(move || {
         disk.refresh();
@@ -71,40 +74,43 @@ fn bench_refresh_disk(b: &mut test::Bencher) {
 
 #[bench]
 fn bench_refresh_disks(b: &mut test::Bencher) {
-    let mut s = sysinfo::System::new_all();
+    let mut disks = sysinfo::Disks::new_with_refreshed_list();
 
     b.iter(move || {
-        s.refresh_disks();
+        disks.refresh();
     });
 }
 
 #[bench]
 fn bench_refresh_disks_list(b: &mut test::Bencher) {
-    let mut s = sysinfo::System::new();
+    let mut disks = sysinfo::Disks::new_with_refreshed_list();
 
     b.iter(move || {
-        s.refresh_disks_list();
+        disks.refresh_list();
     });
 }
 
+#[cfg(feature = "network")]
 #[bench]
 fn bench_refresh_networks(b: &mut test::Bencher) {
-    let mut s = sysinfo::System::new_all();
+    let mut n = sysinfo::Networks::new_with_refreshed_list();
 
     b.iter(move || {
-        s.refresh_networks();
+        n.refresh();
     });
 }
 
+#[cfg(feature = "network")]
 #[bench]
 fn bench_refresh_networks_list(b: &mut test::Bencher) {
-    let mut s = sysinfo::System::new();
+    let mut n = sysinfo::Networks::new_with_refreshed_list();
 
     b.iter(move || {
-        s.refresh_networks_list();
+        n.refresh_list();
     });
 }
 
+#[cfg(feature = "system")]
 #[bench]
 fn bench_refresh_memory(b: &mut test::Bencher) {
     let mut s = sysinfo::System::new();
@@ -114,38 +120,42 @@ fn bench_refresh_memory(b: &mut test::Bencher) {
     });
 }
 
+#[cfg(feature = "system")]
 #[bench]
-fn bench_refresh_cpu(b: &mut test::Bencher) {
+fn bench_refresh_cpu_usage(b: &mut test::Bencher) {
     let mut s = sysinfo::System::new();
 
+    s.refresh_cpu_usage();
     b.iter(move || {
-        s.refresh_cpu();
+        s.refresh_cpu_usage();
     });
 }
 
+#[cfg(feature = "component")]
 #[bench]
 fn bench_refresh_components(b: &mut test::Bencher) {
-    let mut s = sysinfo::System::new_all();
+    let mut c = sysinfo::Components::new_with_refreshed_list();
 
     b.iter(move || {
-        s.refresh_components();
+        c.refresh();
     });
 }
 
+#[cfg(feature = "component")]
 #[bench]
 fn bench_refresh_components_list(b: &mut test::Bencher) {
-    let mut s = sysinfo::System::new_all();
+    let mut c = sysinfo::Components::new_with_refreshed_list();
 
     b.iter(move || {
-        s.refresh_components_list();
+        c.refresh_list();
     });
 }
 
 #[bench]
 fn bench_refresh_users_list(b: &mut test::Bencher) {
-    let mut s = sysinfo::System::new_all();
+    let mut users = sysinfo::Users::new_with_refreshed_list();
 
     b.iter(move || {
-        s.refresh_users_list();
+        users.refresh_list();
     });
 }
